@@ -1,10 +1,11 @@
 from enum import Enum
-from typing import Dict, Optional
+from typing import Dict, Optional, List
 import turing.generated.models
 from turing.generated.model_utils import OpenApiModel
 from turing.generated.model.env_var import EnvVar
 from .source import EnsemblingJobSource, EnsemblingJobPredictionSource
 from .sink import EnsemblingJobSink
+from ...mounted_mlp_secret import MountedMLPSecret
 
 ResourceRequest = turing.generated.models.EnsemblingResources
 
@@ -22,21 +23,15 @@ class ResultType(Enum):
 
 
 class ResultConfig:
-
     def __init__(
-            self,
-            type: ResultType,
-            column_name: str,
-            item_type: Optional[ResultType] = None):
+        self, type: ResultType, column_name: str, item_type: Optional[ResultType] = None
+    ):
         self._type, self._column_name, self._item_type = type, column_name, item_type
 
     def to_open_api(self) -> OpenApiModel:
-        kwargs = {
-            'type': self._type.to_open_api(),
-            'column_name': self._column_name
-        }
+        kwargs = {"type": self._type.to_open_api(), "column_name": self._column_name}
         if self._item_type:
-            kwargs['item_type'] = self._item_type.to_open_api()
+            kwargs["item_type"] = self._item_type.to_open_api()
 
         return turing.generated.models.EnsemblingJobEnsemblerSpecResult(**kwargs)
 
@@ -46,14 +41,17 @@ class EnsemblingJobConfig:
     Configuration of the batch ensembling job
     """
 
-    def __init__(self,
-                 source: EnsemblingJobSource,
-                 predictions: Dict[str, EnsemblingJobPredictionSource],
-                 result_config: ResultConfig,
-                 sink: EnsemblingJobSink,
-                 service_account: str,
-                 resource_request: ResourceRequest = None,
-                 env_vars: Dict[str, str] = {}):
+    def __init__(
+        self,
+        source: EnsemblingJobSource,
+        predictions: Dict[str, EnsemblingJobPredictionSource],
+        result_config: ResultConfig,
+        sink: EnsemblingJobSink,
+        service_account: str,
+        resource_request: ResourceRequest = None,
+        env_vars: Dict[str, str] = {},
+        secrets: List[MountedMLPSecret] = None,
+    ):
         """
         Create new instance of batch ensembling job configuration
 
@@ -65,6 +63,7 @@ class EnsemblingJobConfig:
         :param resource_request: optional resource request for starting the ensembling job.
             If not given the system default will be used.
         :param env_vars: optional environment variables in the form of a key value pair in a list.
+        :param secrets: list of MLP secrets to mount into the ensembling job environment as environment variables
         """
         self._source = source
         self._predictions = predictions
@@ -73,9 +72,10 @@ class EnsemblingJobConfig:
         self._service_account = service_account
         self._resource_request = resource_request
         self._env_vars = env_vars
+        self._secrets = secrets
 
     @property
-    def source(self) -> 'EnsemblingJobSource':
+    def source(self) -> "EnsemblingJobSource":
         return self._source
 
     @property
@@ -83,11 +83,11 @@ class EnsemblingJobConfig:
         return self._predictions
 
     @property
-    def sink(self) -> 'EnsemblingJobSink':
+    def sink(self) -> "EnsemblingJobSink":
         return self._sink
 
     @property
-    def result_config(self) -> 'ResultConfig':
+    def result_config(self) -> "ResultConfig":
         return self._result_config
 
     @property
@@ -99,27 +99,39 @@ class EnsemblingJobConfig:
         return self._env_vars
 
     @property
-    def resource_request(self) -> Optional['ResourceRequest']:
+    def secrets(self) -> Optional[List[MountedMLPSecret]]:
+        return self._secrets
+
+    @property
+    def resource_request(self) -> Optional["ResourceRequest"]:
         return self._resource_request
 
     def job_spec(self) -> turing.generated.models.EnsemblingJobSpec:
         return turing.generated.models.EnsemblingJobSpec(
             source=self.source.to_open_api(),
-            predictions={name: source.to_open_api() for name, source in self.predictions.items()},
+            predictions={
+                name: source.to_open_api() for name, source in self.predictions.items()
+            },
             ensembler=turing.generated.models.EnsemblingJobEnsemblerSpec(
                 result=self.result_config.to_open_api()
             ),
-            sink=self.sink.to_open_api()
+            sink=self.sink.to_open_api(),
         )
 
     def infra_spec(self) -> turing.generated.models.EnsemblerInfraConfig:
         if self.env_vars is None:
             env_vars = []
         else:
-            env_vars = [EnvVar(name=name, value=value) for name, value in self.env_vars.items()]\
-
+            env_vars = [
+                EnvVar(name=name, value=value) for name, value in self.env_vars.items()
+            ]
         return turing.generated.models.EnsemblerInfraConfig(
             service_account_name=self.service_account,
             resources=self.resource_request,
             env=env_vars,
+            secrets=(
+                [secret.to_open_api() for secret in self.secrets]
+                if self.secrets
+                else []
+            ),
         )
