@@ -211,6 +211,18 @@ class RouterConfig:
             self._experiment_engine = experiment_engine
         elif isinstance(experiment_engine, dict):
             self._experiment_engine = ExperimentConfig(**experiment_engine)
+            # This block sets the passkey (encrypted when retrieved from the Turing API server) to None to prevent users
+            # from sending a router config with an already encrypted passkey back to API server when updating a router.
+            #
+            # When the passkey value is not set, the Turing API server is able to automatically retrieve the correct
+            # passkey from the existing router version (assuming it has been configured with the same standard
+            # experiment engine and the same client username).
+            if (
+                self._experiment_engine.config is not None
+                and self._experiment_engine.config.get("client") is not None
+                and self._experiment_engine.config["client"].get("passkey") != ""
+            ):
+                self._experiment_engine.config["client"]["passkey"] = None
         else:
             self._experiment_engine = experiment_engine
 
@@ -277,9 +289,6 @@ class RouterConfig:
             self._log_config = LogConfig(**log_config)
         else:
             self._log_config = log_config
-        # Verify that only nop or UPI logger is configured for UPI router
-        if self._protocol == Protocol.UPI:
-            self._verify_upi_router_logger()
 
     @property
     def enricher(self) -> Enricher:
@@ -422,16 +431,6 @@ class RouterConfig:
                 f"UPI router only supports no ensembler or standard ensembler."
             )
 
-    def _verify_upi_router_logger(self):
-        # only nop or upi logger type is allowed
-        if not (
-            self.log_config.result_logger_type == ResultLoggerType.NOP
-            or self.log_config.result_logger_type == ResultLoggerType.UPI
-        ):
-            raise turing.router.config.log_config.InvalidResultLoggerTypeAndConfigCombination(
-                f"UPI router only supports no logging or UPI logger"
-            )
-
     def to_dict(self):
         att_dict = {}
         for m in inspect.getmembers(self):
