@@ -1,27 +1,41 @@
 import os
-import mlflow
+from time import sleep
 from typing import List, Optional
 
+import mlflow
+import pyprind
+
 import turing.generated.models
 from turing.ensembler import EnsemblerType
 from turing.generated import ApiClient, Configuration
-from turing.generated.apis import EnsemblerApi, EnsemblingJobApi, ProjectApi, RouterApi
+from turing.generated.apis import (
+    EnsemblerApi,
+    EnsemblerImagesApi,
+    EnsemblingJobApi,
+    ProjectApi,
+    RouterApi,
+)
+from turing.generated.model.ensembler_image_runner_type import EnsemblerImageRunnerType
 from turing.generated.models import (
-    Project,
     Ensembler,
-    EnsemblingJob,
+    EnsemblerId,
+    EnsemblerImage,
+    EnsemblerImages,
     EnsemblerJobStatus,
     EnsemblersPaginatedResults,
+    EnsemblingJob,
     EnsemblingJobPaginatedResults,
     JobId,
-    RouterId,
-    RouterIdObject,
-    RouterIdAndVersion,
+    Project,
     Router,
-    RouterDetails,
     RouterConfig,
+    RouterDetails,
+    RouterId,
+    RouterIdAndVersion,
+    RouterIdObject,
     RouterVersion,
     RouterVersionConfig,
+    RouterVersionStatus,
 )
 
 
@@ -39,8 +53,6 @@ class TuringSession:
     Session object for interacting with Turing back-end
     """
 
-    OAUTH_SCOPES = ["https://www.googleapis.com/auth/userinfo.email"]
-
     def __init__(
         self, host: str, project_name: str = None, use_google_oauth: bool = True
     ):
@@ -55,9 +67,11 @@ class TuringSession:
         self._api_client = ApiClient(config)
 
         if use_google_oauth:
-            from caraml_auth.id_token_credentials import get_default_id_token_credentials
+            from caraml_auth.id_token_credentials import (
+                get_default_id_token_credentials,
+            )
             from google.auth.transport.requests import Request
-            from google.auth.transport.urllib3 import urllib3, AuthorizedHttp
+            from google.auth.transport.urllib3 import AuthorizedHttp, urllib3
 
             credentials = get_default_id_token_credentials(target_audience="sdk.caraml")
             # Refresh credentials, in case it's coming from Compute Engine.
@@ -172,12 +186,104 @@ class TuringSession:
             ensembler=ensembler,
         )
 
+    @require_active_project
+    def delete_ensembler(self, ensembler_id: int) -> EnsemblerId:
+        """
+        Delete Ensembler with Ensembler Id
+        """
+        return EnsemblerApi(self._api_client).delete_ensembler(
+            project_id=self.active_project.id, ensembler_id=ensembler_id
+        )
+
+    @require_active_project
+    def list_ensembler_images(
+        self,
+        ensembler: Ensembler,
+        runner_type: EnsemblerImageRunnerType = None,
+    ) -> EnsemblerImages:
+        """
+        List ensembler images
+        """
+        return EnsemblerImagesApi(self._api_client).list_ensembler_images(
+            project_id=self.active_project.id,
+            ensembler_id=ensembler.id,
+            runner_type=runner_type,
+        )
+
+    @require_active_project
+    def create_ensembler_image(
+        self,
+        ensembler: Ensembler,
+        runner_type: EnsemblerImageRunnerType = None,
+    ) -> EnsemblerImage:
+        """
+        Create ensembler image
+        """
+        build_ensembler_image_request = (
+            turing.generated.models.BuildEnsemblerImageRequest(
+                runner_type=runner_type,
+            )
+        )
+
+        EnsemblerImagesApi(self._api_client).create_ensembler_image(
+            project_id=self.active_project.id,
+            ensembler_id=ensembler.id,
+            build_ensembler_image_request=build_ensembler_image_request,
+        )
+
+        bar = pyprind.ProgBar(
+            100,
+            track_time=True,
+            title=f"Building Docker image for ensembler {runner_type} of {ensembler.name}",
+        )
+        bar.update()
+        sleep(10)
+
+        while bar.active:
+            images = self.list_ensembler_images(ensembler, runner_type)
+
+            if len(images.value) != 1:
+                break
+
+            image = images.value[0]
+
+            if image.exists:
+                break
+
+            if (
+                image.image_building_job_status is not None
+                and image.image_building_job_status.state.value != "active"
+            ):
+                break
+
+            bar.update()
+            sleep(10)
+        bar.stop()
+
+        if image.exists:
+            print(
+                f"Succefully built Docker image for ensembler {runner_type} of {ensembler.name}."
+                f"\nDocker image ref: {image.image_ref}"
+            )
+        else:
+            print(
+                f"Failed to build Docker image for model {runner_type} of {ensembler.name}"
+            )
+            if (
+                image.image_building_job_status is not None
+                and image.image_building_job_status.message != ""
+            ):
+                print(f"{image.image_building_job_status.message}")
+
+        return image
+
     @require_active_project
     def list_ensembling_jobs(
         self,
         status: List[EnsemblerJobStatus] = None,
         page: Optional[int] = None,
         page_size: Optional[int] = None,
+        ensembler_id: Optional[int] = None,
     ) -> EnsemblingJobPaginatedResults:
         """
         List ensembling jobs
@@ -190,6 +296,8 @@ class TuringSession:
             kwargs["page"] = page
         if page_size:
             kwargs["page_size"] = page_size
+        if ensembler_id:
+            kwargs["ensembler_id"] = ensembler_id
 
         return EnsemblingJobApi(self._api_client).list_ensembling_jobs(
             project_id=self.active_project.id, **kwargs
@@ -356,3 +464,27 @@ class TuringSession:
         ).projects_project_id_routers_router_id_events_get(
             project_id=self.active_project.id, router_id=router_id
         )
+
+    @require_active_project
+    def list_router_versions_with_filter(
+        self,
+        ensembler_id: int = None,
+        status: List[RouterVersionStatus] = None,
+        is_current: bool = False,
+    ) -> RouterVersion:
+        """
+        Fetch specific router version by its router ID and version
+        """
+        kwargs = {}
+
+        if ensembler_id:
+            kwargs["ensembler_id"] = ensembler_id
+
+        if status:
+            kwargs["status"] = status
+
+        kwargs["is_current"] = is_current
+
+        return RouterApi(self._api_client).projects_project_id_router_versions_get(
+            project_id=self.active_project.id, **kwargs
+        )
